<?php namespace Everyware\Plugin\GoogleAnalytics;

use Infomaker\Everyware\Base\Admin\Form;
use Infomaker\Everyware\Base\JsonImporter;
use Infomaker\Everyware\Base\Message;
use Infomaker\Everyware\Base\MessageBag;
use Infomaker\Everyware\Support\Str;
use Infomaker\Everyware\Twig\View;
use Infomaker\ProjectPlugin\Interfaces\SettingsPageInterface;

class AdminTab implements SettingsPageInterface
{

    private static $title = 'Google Analytics';

    private static $settingsSlug = 'ew-google-analytics-settings';

    /**
     * Twig-template to use
     *
     * @var string
     */
    protected static $template = '@plugins/everyware-plugin-google-analytics/src/templates/admin';

    /**
     * @var JsonImporter
     */
    private $jsonImporter;

    /**
     * @var Form
     */
    private $formHandler;

    /**
     * Keep track if error has occurred
     *
     * @var bool
     */
    private $hasError = false;

    /**
     * @var Settings
     */
    private $settings;

    public function __construct(Settings $settings)
    {
        $this->formHandler = new Form(static::$settingsSlug, $settings->getFields());
        $this->settings = $settings;
    }

    /**
     * Assemble data to be sent to the template
     *
     * @return array
     */
    private function createSettingsData(): array
    {
        $importedSettings = $this->jsonImporter->getJson();
        $settings = $this->settings->get();

        return array_merge($this->formHandler->getFormData($settings), [
            'preview_credentials' => $this->extractPreviewCredentials(),
            'imported_credentials' => $importedSettings,
            'json_import' => $this->generateJsonImporter(json_encode($importedSettings),
                $this->settings->getCredentials()->toJson(JSON_UNESCAPED_SLASHES|JSON_PRETTY_PRINT)),
        ]);
    }

    /**
     * @param string $import
     * @param string $export
     *
     * @return string
     */
    private function generateJsonImporter(string $import, string $export): string
    {
        return View::generate('@projectPlugin/utilities/import-export', [
            'field_name' => JsonImporter::POST_KEY,
            'import' => $import ?? '',
            'export' => $export ?? '',
        ]);
    }

    /**
     * Fires on Page-load if active
     *
     * @return string
     */
    public function currentAction(): string
    {
        return $_POST['action_type'] ?? '';
    }

    private function currentSettings()
    {
        return $_POST[static::$settingsSlug] ?? [];
    }

    /**
     * Retrieve any messages that may be displayed by the settings-page
     *
     * @return string
     */
    public function getStatusMessage(): string
    {
        if ($this->isUpdated()) {
            Message::create('Success', 'Updated! Changes have been saved.');
        } elseif ($this->jsonImporter->isImported()) {
            Message::create('Success', 'Import done!');
        }

        return MessageBag::messagesToHtml();
    }

    /**
     * Retrieve the slug to be used to identify and activate tab
     *
     * @return string
     */
    public function getTabSlug(): string
    {
        return Str::slug(static::$title);
    }

    /**
     * The title/label displayed on the tab
     *
     * @return string
     */
    public function getTabTitle(): string
    {
        return static::$title;
    }

    protected function isEdit(): bool
    {
        return $_POST['action_type'] ?? '' === 'edit';
    }

    /**
     * Determine if form has been submitted
     *
     * @return bool
     */
    protected function isUpdated(): bool
    {
        return $this->isEdit();
    }

    /**
     * Fires on Page-load if active
     *
     * @return void
     */
    public function onPageLoad(): void
    {
        $this->jsonImporter = new JsonImporter();
        if ( ! $this->routeAction($this->currentAction())) {
            $this->hasError = true;
        }
    }

    public function onError(): bool
    {
        return $this->hasError;
    }

    /**
     * Fires on Page-load if active
     *
     * @return string
     */
    public function pageContent(): string
    {
        return View::generate(static::$template, $this->createSettingsData());
    }

    /**
     * @param $action
     *
     * @return bool
     */
    private function routeAction($action): bool
    {
        if ($action === 'save') {
            return $this->saveSettings($this->currentSettings());
        }

        if ($this->jsonImporter->isImported()) {
            $this->settings->updateCredentials($this->jsonImporter->getJson());
        }

        return false;
    }

    /**
     * @param $settings
     *
     * @return bool
     */
    private function saveSettings($settings): bool
    {
        return $this->settings->update($settings);
    }

    private function extractPreviewCredentials(): array
    {
        $credentials = $this->settings->getCredentials();

        return [
            'project_id' => $credentials->project_id,
            'client_email' => $credentials->client_email,
            'client_id' => $credentials->client_id
        ];
    }
}
