<?php namespace Everyware\Plugin\GoogleAnalytics;

use Everyware\Plugin\GoogleAnalytics\Models\Credentials;
use Infomaker\Everyware\Support\Storage\ObjectDB;
use Infomaker\Everyware\Support\Str;

/**
 * Settings
 *
 * @link    http://infomaker.se
 * @package Everyware\Plugin\GoogleAnalytics
 * @since   Everyware\Plugin\GoogleAnalytics\Settings 1.0.0
 */
class Settings
{

    /**
     * @var string
     */
    protected static $settingsTable = 'ew_google_analytics_settings';

    /**
     * @var array
     */
    protected static $defaultSettings = [
        'credentials' => [],
        'time_start' => 'yesterday',
        'time_stop' => 'today',
        'tracking_id' => '',
        'view_id' => ''
    ];

    /**
     * Contains the settings after first fetch
     *
     * @var
     */
    protected static $settings;

    /**
     * Short way of retrieving the tracking ID for Google Analytics
     *
     * @since 1.0.0
     * @return mixed
     */
    public static function getTrackingId()
    {
        return (new static())->getValue('tracking_id');
    }

    public function getFields(): array
    {
        return static::$defaultSettings;
    }

    public function update(array $settings = []): bool
    {
        // Make sure credentials are not removed
        if ($settings['credentials'] ?? '' === '{imported}') {
            $settings['credentials'] = $this->getValue('credentials', []);
        }

        $success = ObjectDB::update(static::$settingsTable, $settings);

        // Save static settings with new settings
        if ($success) {
            static::$settings = $settings;
        }

        return $success;
    }

    public function get(): array
    {
        if (static::$settings === null) {
            static::$settings = ObjectDB::get(static::$settingsTable, static::$defaultSettings);
        }

        return static::$settings;
    }

    /**
     * Retrieve a value of a given settings key
     *
     * @param string $key
     * @param string $default
     *
     * @return mixed
     */
    public function getValue($key, $default = '')
    {
        $settings = $this->get();

        if ( ! array_key_exists($key, $settings)) {
            return $default;
        }

        return $settings[$key];
    }

    /**
     * @param array $credentials
     *
     * @return bool
     */
    public function updateCredentials(array $credentials = []): bool
    {
        $newCredentials = new Credentials($credentials);
        $settings = $this->get();
        $settings['credentials'] = $newCredentials->toArray();

        return $this->update($settings);
    }

    public function getCredentials(): Credentials
    {
        return new Credentials($this->getValue('credentials', []));
    }

    public function getTimeSpan(): array
    {
        return [
            'from' => $this->getValue('time_start'),
            'to' => $this->getValue('time_stop')
        ];
    }

    public function hasViewId(): bool
    {
        return Str::notEmpty($this->getViewId());
    }

    /**
     * @since 1.0.0
     * @return mixed
     */
    public function getViewId()
    {
        return $this->getValue('view_id');
    }
}
